function [p1] = kinModel(p0, u, dt)
    % Kinematic Model for Diferential Drive
    %
    % Equation
    %    dx/dt = cos(phi)*v
    %    dy/dt = sin(phi)*v
    %    dphi/dt = w
    %
    % Syntax:
    %       [p1,lR,lL] = kinModel(p0, u, dt)
    %
    % Input:
    %   p0: Actual Position of the vehicle, [x y phi]^T
    %   u:  Input signal, [v w]^T
    %   dt: Size of the time step
    %
    % Output:
    %   p1:     Position after movement
    %   lR,lL:  Movement of the right and left wheel
    %
    % Date:     17.05.2018
    % Author:   Nils Rottmann (Nils.Rottmann@rob.uni-luebeck.de)
    
    %% Check for correct dimensions
    if (size(p0) ~= [3 1])
        error('Size of input p0 is not correct!')
    end
    if (size(u) ~= [2 1])
        error('Size of input u is not correct!')
    end
    if (size(dt) ~= [1 1])
        error('Size of input dt is not correct!')
    end

    %% Parameters
    L = 0.15;           % half of the dimension between axes, in [m] 
    v_max = 0.5;        % maximal speed in [m/s]
    w_max = 0.5;        % maximal angular velocity in [m/s]
    
    
    %% Kinematic Model from "Probabilistic Robotics"
    v = u(1);
    w = u(2);
   
    % Check boundaries
    if v > v_max; v = v_max; end
    if v < -v_max; v = -v_max; end
    if w > w_max; w = w_max; end
    if w < -w_max; w = -w_max; end
    
    % Handle singularity
    if (abs(w) < 10^(-6))
        phi0 = p0(3); % + g*dt;
        p1 = p0 + [v*cos(phi0)*dt; v*sin(phi0)*dt; 0];
        w = 0;
    else
        phi0 = p0(3);
        phi1 = phi0 + w*dt;
        vw = v/w;
        
        p1 = zeros(3,1);
        p1(1) = p0(1) + vw*(sin(phi1)-sin(phi0));
        p1(2) = p0(2) + vw*(cos(phi0)-cos(phi1));
        p1(3) = phi1; % + g*dt;   
    end

end